<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Infomaker\Everyware\Support\NewRelicLog;
use Twig\Environment;
use Twig\Util\DeprecationCollector;

/**
 * Class DeprecationHandler
 * @package Infomaker\Everyware\Twig
 */
class DeprecationHandler
{
    /**
     * @var DeprecationCollector
     */
    private $collector;

    /**
     * @var array
     */
    private $registeredPaths = [];

    public function __construct(Environment $twig)
    {
        $this->collector = new DeprecationCollector($twig);
    }

    public function search(\Twig\Loader\FilesystemLoader $loader): array
    {
        $paths = $this->getRegisteredPaths($loader);
        $deprecations = array_map([$this->collector, 'collectDir'], $paths);
        return array_merge(...$deprecations);
    }

    public function searchAndLog(\Twig\Loader\FilesystemLoader $loader)
    {
        $deprecationWarnings = $this->search($loader);

        foreach ($deprecationWarnings as $warning) {
            NewRelicLog::error($warning);
        }
    }

    public function listen(): void
    {
        set_error_handler(static function ($type, $msg) {
            if (E_USER_DEPRECATED === $type) {
                NewRelicLog::error($msg);
            }
        });
    }

    private function getRegisteredPaths(\Twig\Loader\FilesystemLoader $loader): array
    {
        $paths = [[]];
        foreach ($loader->getNamespaces() as $namespace) {
            $paths[] = $this->registeredPaths[$namespace] = $loader->getPaths($namespace);
        }

        return array_merge(...$paths);
    }
}
