<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Twig\Environment;
use Twig\Extension\ExtensionInterface;
use Twig\NodeVisitor\NodeVisitorInterface;
use Twig\RuntimeLoader\RuntimeLoaderInterface;
use Twig\TokenParser\TokenParserInterface;
use Twig\TwigFilter;
use Twig\TwigFunction;
use Twig\TwigTest;

/**
 * Class TwigExtender
 * @package Infomaker\Everyware\Twig
 */
class TwigExtender implements TwigExtendable
{
    /**
     * @var Environment
     */
    private $environment;

    public function __construct(Environment $environment)
    {
        $this->environment = $environment;
    }

    public function addExtension(ExtensionInterface $extension): void
    {
        $this->environment->addExtension($extension);
    }

    public function addFilter(TwigFilter $filter): void
    {
        $this->environment->addFilter($filter);
    }

    public function addFunction(TwigFunction $function): void
    {
        $this->environment->addFunction($function);
    }

    public function addGlobal(string $name, $value): void
    {
        $this->environment->addGlobal($name, $value);
    }

    public function addNodeVisitor(NodeVisitorInterface $visitor): void
    {
        $this->environment->addNodeVisitor($visitor);
    }

    public function addTest(TwigTest $test): void
    {
        $this->environment->addTest($test);
    }

    public function addTokenParser(TokenParserInterface $parser): void
    {
        $this->environment->addTokenParser($parser);
    }

    public function hasExtension(string $class): bool
    {
        return $this->environment->hasExtension($class);
    }

    public function addRuntimeLoader(RuntimeLoaderInterface $loader): void
    {
        $this->environment->addRuntimeLoader($loader);
    }
}
