<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Twig\Environment;
use Twig\Extension\ExtensionInterface;
use Twig\NodeVisitor\NodeVisitorInterface;
use Twig\RuntimeLoader\RuntimeLoaderInterface;
use Twig\TokenParser\TokenParserInterface;
use Twig\TwigFilter;
use Twig\TwigFunction;
use Twig\TwigTest;

/**
 * Class TwigEnvironmentExtenderStorage
 * @package Infomaker\Everyware\Twig
 */
class TwigExtenderStorage implements TwigExtendable, TwigEnvironmentExtender
{
    private $extensions = [];
    private $filters = [];
    private $functions = [];
    private $globals = [];
    private $tests = [];
    private $visitors = [];
    private $parsers = [];
    private $loaders = [];

    public function addExtension(ExtensionInterface $extension): void
    {
        $this->extensions[get_class($extension)] = $extension;
    }

    public function addFilter(TwigFilter $filter): void
    {
        $this->filters[$filter->getName()] = $filter;
    }

    public function addFunction(TwigFunction $function): void
    {
        $this->functions[$function->getName()] = $function;
    }

    public function addGlobal(string $name, $value): void
    {
        $this->globals[$name] = $value;
    }

    public function addNodeVisitor(NodeVisitorInterface $visitor): void
    {
        $this->visitors[get_class($visitor)] = $visitor;
    }

    public function addTest(TwigTest $test): void
    {
        $this->tests[$test->getName()] = $test->getName();
    }

    public function addTokenParser(TokenParserInterface $parser): void
    {
        $this->parsers[get_class($parser)] = $parser;
    }

    public function hasExtension(string $class): bool
    {
        return array_key_exists($class, $this->extensions);
    }

    public function addRuntimeLoader(RuntimeLoaderInterface $loader): void
    {
        $this->loaders[get_class($loader)] = $loader;
    }

    public function extendEnvironment(Environment $twig): Environment
    {
        foreach ($this->extensions as $extension) {
            $twig->addExtension($extension);
        }

        foreach ($this->filters as $filter) {
            $twig->addFilter($filter);
        }

        foreach ($this->functions as $function) {
            $twig->addFunction($function);
        }

        foreach ($this->globals as $key => $value) {
            $twig->addGlobal($key, $value);
        }

        foreach ($this->visitors as $visitor) {
            $twig->addNodeVisitor($visitor);
        }

        foreach ($this->tests as $test) {
            $twig->addTest($test);
        }

        foreach ($this->parsers as $parser) {
            $twig->addTokenParser($parser);
        }

        foreach ($this->loaders as $loader) {
            $twig->addRuntimeLoader($loader);
        }

        return $twig;
    }
}
